// Copyright (C) 2018 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only

#include "qwlhardwarelayerintegration_p.h"

QT_BEGIN_NAMESPACE

namespace QtWayland {

}

QT_END_NAMESPACE

#include "moc_qwlhardwarelayerintegration_p.cpp"
