// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#include "qwaylandclientbufferintegration_p.h"

QT_BEGIN_NAMESPACE

namespace QtWaylandClient {

QWaylandClientBufferIntegration::QWaylandClientBufferIntegration()
{

}

QWaylandClientBufferIntegration::~QWaylandClientBufferIntegration()
{

}

}

QT_END_NAMESPACE
