// Copyright (C) 2017 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only

#include "qwlclientbufferintegration_p.h"

QT_BEGIN_NAMESPACE

namespace QtWayland {

ClientBufferIntegration::ClientBufferIntegration()
{
}

}

QT_END_NAMESPACE
